@@ -1,9 +1,3 @@
-#
-# Created by: Pearu Peterson, March 2002
-#
-""" Test functions for linalg.basic module
-
-"""
 from __future__ import division, print_function, absolute_import
 
 import warnings
@@ -24,28 +18,12 @@ from scipy._lib._numpy_compat import suppress_warnings
 from scipy.linalg import (solve, inv, det, lstsq, pinv, pinv2, pinvh, norm,
                           solve_banded, solveh_banded, solve_triangular,
                           solve_circulant, circulant, LinAlgError, block_diag,
-                          matrix_balance)
+                          matrix_balance, LinAlgWarning)
 
-from scipy.linalg.basic import LstsqLapackError
 from scipy.linalg._testutils import assert_no_overwrite
 
 from scipy._lib._version import NumpyVersion
 
-
-"""
-Bugs:
-1) solve.check_random_sym_complex fails if a is complex
-   and transpose(a) = conjugate(a) (a is Hermitian).
-"""
-__usage__ = """
-Build linalg:
-  python setup_linalg.py build
-Run tests if scipy is installed:
-  python -c 'import scipy;scipy.linalg.test()'
-Run tests if linalg is not installed:
-  python tests/test_basic.py
-"""
-
 REAL_DTYPES = [np.float32, np.float64, np.longdouble]
 COMPLEX_DTYPES = [np.complex64, np.complex128, np.clongdouble]
 DTYPES = REAL_DTYPES + COMPLEX_DTYPES
@@ -611,7 +589,6 @@ class TestSolve(object):
     def test_random_sym_complex(self):
         n = 20
         a = random([n, n])
-        # XXX: with the following addition the accuracy will be very low
         a = a + 1j*random([n, n])
         for i in range(n):
             a[i, i] = abs(20*(.1+a[i, i]))
@@ -713,7 +690,7 @@ class TestSolve(object):
         b = np.ones(2)
         with warnings.catch_warnings():
             warnings.simplefilter('error')
-            assert_raises(RuntimeWarning, solve, a, b)
+            assert_raises(LinAlgWarning, solve, a, b)
 
     def test_empty_rhs(self):
         a = np.eye(2)
@@ -753,11 +730,10 @@ class TestSolve(object):
     @pytest.mark.skip(reason="Failure on OS X (gh-7500), "
                              "crash on Windows (gh-8064)")
     def test_all_type_size_routine_combinations(self):
-        sizes = [10, 100, 1000]
+        sizes = [10, 100]
         assume_as = ['gen', 'sym', 'pos', 'her']
         dtypes = [np.float32, np.float64, np.complex64, np.complex128]
-        for size, assume_a, dtype in itertools.product(sizes,
-                                                       assume_as,
+        for size, assume_a, dtype in itertools.product(sizes, assume_as,
                                                        dtypes):
             is_complex = dtype in (np.complex64, np.complex128)
             if assume_a == 'her' and not is_complex:
@@ -792,7 +768,8 @@ class TestSolve(object):
                             rtol=tol * size,
                             err_msg=err_msg)
 
-            if assume_a == 'sym' and dtype not in (np.complex64, np.complex128):
+            if assume_a == 'sym' and dtype not in (np.complex64,
+                                                   np.complex128):
                 x = solve(a, b, assume_a=assume_a, transposed=True)
                 assert_allclose(a.dot(x), b,
                                 atol=tol * size,
@@ -832,6 +809,26 @@ class TestSolveTriangular(object):
         sol = solve_triangular(A, b, lower=True, trans=1)
         assert_array_almost_equal(sol, [[.5-.5j, -.25-.25j], [0, 0.5]])
 
+        # check other option combinations with complex rhs
+        b = np.diag([1+1j, 1+2j])
+        sol = solve_triangular(A, b, lower=True, trans=0)
+        assert_array_almost_equal(sol, [[1, 0], [-0.5j, 0.5+1j]])
+
+        sol = solve_triangular(A, b, lower=True, trans=1)
+        assert_array_almost_equal(sol, [[1, 0.25-0.75j], [0, 0.5+1j]])
+
+        sol = solve_triangular(A, b, lower=True, trans=2)
+        assert_array_almost_equal(sol, [[1j, -0.75-0.25j], [0, 0.5+1j]])
+
+        sol = solve_triangular(A.T, b, lower=False, trans=0)
+        assert_array_almost_equal(sol, [[1, 0.25-0.75j], [0, 0.5+1j]])
+
+        sol = solve_triangular(A.T, b, lower=False, trans=1)
+        assert_array_almost_equal(sol, [[1, 0], [-0.5j, 0.5+1j]])
+
+        sol = solve_triangular(A.T, b, lower=False, trans=2)
+        assert_array_almost_equal(sol, [[1j, 0], [-0.5, 0.5+1j]])
+
     def test_check_finite(self):
         """
         solve_triangular on a simple 2x2 matrix.
@@ -943,37 +940,26 @@ class TestLstsq(object):
         for dtype in REAL_DTYPES:
             a = np.array([[1, 20], [-30, 4]], dtype=dtype)
             for lapack_driver in TestLstsq.lapack_drivers:
-                    for overwrite in (True, False):
-                        for bt in (((1, 0), (0, 1)), (1, 0),
-                                   ((2, 1), (-30, 4))):
-                            # Store values in case they are overwritten
-                            # later
-                            a1 = a.copy()
-                            b = np.array(bt, dtype=dtype)
-                            b1 = b.copy()
-                            try:
-                                out = lstsq(a1, b1,
-                                            lapack_driver=lapack_driver,
-                                            overwrite_a=overwrite,
-                                            overwrite_b=overwrite)
-                            except LstsqLapackError:
-                                if lapack_driver is None:
-                                    mesg = ('LstsqLapackError raised with '
-                                            'lapack_driver being None.')
-                                    raise AssertionError(mesg)
-                                else:
-                                    # can't proceed, skip to the next iteration
-                                    continue
-
-                            x = out[0]
-                            r = out[2]
-                            assert_(r == 2,
-                                    'expected efficient rank 2, got %s' % r)
-                            assert_allclose(
-                                          dot(a, x), b,
-                                          atol=25 * _eps_cast(a1.dtype),
-                                          rtol=25 * _eps_cast(a1.dtype),
-                                          err_msg="driver: %s" % lapack_driver)
+                for overwrite in (True, False):
+                    for bt in (((1, 0), (0, 1)), (1, 0),
+                               ((2, 1), (-30, 4))):
+                        # Store values in case they are overwritten
+                        # later
+                        a1 = a.copy()
+                        b = np.array(bt, dtype=dtype)
+                        b1 = b.copy()
+                        out = lstsq(a1, b1,
+                                    lapack_driver=lapack_driver,
+                                    overwrite_a=overwrite,
+                                    overwrite_b=overwrite)
+                        x = out[0]
+                        r = out[2]
+                        assert_(r == 2,
+                                'expected efficient rank 2, got %s' % r)
+                        assert_allclose(dot(a, x), b,
+                                        atol=25 * _eps_cast(a1.dtype),
+                                        rtol=25 * _eps_cast(a1.dtype),
+                                        err_msg="driver: %s" % lapack_driver)
 
     def test_simple_overdet(self):
         for dtype in REAL_DTYPES:
@@ -984,19 +970,9 @@ class TestLstsq(object):
                     # Store values in case they are overwritten later
                     a1 = a.copy()
                     b1 = b.copy()
-                    try:
-                        out = lstsq(a1, b1, lapack_driver=lapack_driver,
-                                    overwrite_a=overwrite,
-                                    overwrite_b=overwrite)
-                    except LstsqLapackError:
-                        if lapack_driver is None:
-                            mesg = ('LstsqLapackError raised with '
-                                    'lapack_driver being None.')
-                            raise AssertionError(mesg)
-                        else:
-                            # can't proceed, skip to the next iteration
-                            continue
-
+                    out = lstsq(a1, b1, lapack_driver=lapack_driver,
+                                overwrite_a=overwrite,
+                                overwrite_b=overwrite)
                     x = out[0]
                     if lapack_driver == 'gelsy':
                         residuals = np.sum((b - a.dot(x))**2)
@@ -1023,18 +999,9 @@ class TestLstsq(object):
                     # Store values in case they are overwritten later
                     a1 = a.copy()
                     b1 = b.copy()
-                    try:
-                        out = lstsq(a1, b1, lapack_driver=lapack_driver,
-                                    overwrite_a=overwrite,
-                                    overwrite_b=overwrite)
-                    except LstsqLapackError:
-                        if lapack_driver is None:
-                            mesg = ('LstsqLapackError raised with '
-                                    'lapack_driver being None.')
-                            raise AssertionError(mesg)
-                        else:
-                            # can't proceed, skip to the next iteration
-                            continue
+                    out = lstsq(a1, b1, lapack_driver=lapack_driver,
+                                overwrite_a=overwrite,
+                                overwrite_b=overwrite)
 
                     x = out[0]
                     if lapack_driver == 'gelsy':
@@ -1065,18 +1032,9 @@ class TestLstsq(object):
                     # Store values in case they are overwritten later
                     a1 = a.copy()
                     b1 = b.copy()
-                    try:
-                        out = lstsq(a1, b1, lapack_driver=lapack_driver,
-                                    overwrite_a=overwrite,
-                                    overwrite_b=overwrite)
-                    except LstsqLapackError:
-                        if lapack_driver is None:
-                            mesg = ('LstsqLapackError raised with '
-                                    'lapack_driver being None.')
-                            raise AssertionError(mesg)
-                        else:
-                            # can't proceed, skip to the next iteration
-                            continue
+                    out = lstsq(a1, b1, lapack_driver=lapack_driver,
+                                overwrite_a=overwrite,
+                                overwrite_b=overwrite)
 
                     x = out[0]
                     r = out[2]
@@ -1100,19 +1058,10 @@ class TestLstsq(object):
                             # Store values in case they are overwritten later
                             a1 = a.copy()
                             b1 = b.copy()
-                            try:
-                                out = lstsq(a1, b1,
-                                            lapack_driver=lapack_driver,
-                                            overwrite_a=overwrite,
-                                            overwrite_b=overwrite)
-                            except LstsqLapackError:
-                                if lapack_driver is None:
-                                    mesg = ('LstsqLapackError raised with '
-                                            'lapack_driver being None.')
-                                    raise AssertionError(mesg)
-                                else:
-                                    # can't proceed, skip to the next iteration
-                                    continue
+                            out = lstsq(a1, b1,
+                                        lapack_driver=lapack_driver,
+                                        overwrite_a=overwrite,
+                                        overwrite_b=overwrite)
                             x = out[0]
                             r = out[2]
                             assert_(r == n, 'expected efficient rank %s, '
@@ -1177,20 +1126,10 @@ class TestLstsq(object):
                             # Store values in case they are overwritten later
                             a1 = a.copy()
                             b1 = b.copy()
-                            try:
-                                out = lstsq(a1, b1,
-                                            lapack_driver=lapack_driver,
-                                            overwrite_a=overwrite,
-                                            overwrite_b=overwrite)
-                            except LstsqLapackError:
-                                if lapack_driver is None:
-                                    mesg = ('LstsqLapackError raised with '
-                                            'lapack_driver being None.')
-                                    raise AssertionError(mesg)
-                                else:
-                                    # can't proceed, skip to the next iteration
-                                    continue
-
+                            out = lstsq(a1, b1,
+                                        lapack_driver=lapack_driver,
+                                        overwrite_a=overwrite,
+                                        overwrite_b=overwrite)
                             x = out[0]
                             r = out[2]
                             assert_(r == m, 'expected efficient rank %s, '
@@ -1204,31 +1143,31 @@ class TestLstsq(object):
     def test_random_complex_overdet(self):
         for dtype in COMPLEX_DTYPES:
             for (n, m) in ((20, 15), (200, 2)):
-                    for lapack_driver in TestLstsq.lapack_drivers:
-                        for overwrite in (True, False):
-                            a = np.asarray(random([n, m]) + 1j*random([n, m]),
-                                           dtype=dtype)
-                            for i in range(m):
-                                a[i, i] = 20 * (0.1 + a[i, i])
-                            for i in range(2):
-                                b = np.asarray(random([n, 3]), dtype=dtype)
-                                # Store values in case they are overwritten
-                                # later
-                                a1 = a.copy()
-                                b1 = b.copy()
-                                out = lstsq(a1, b1,
-                                            lapack_driver=lapack_driver,
-                                            overwrite_a=overwrite,
-                                            overwrite_b=overwrite)
-                                x = out[0]
-                                r = out[2]
-                                assert_(r == m, 'expected efficient rank %s, '
-                                        'got %s' % (m, r))
-                                assert_allclose(
-                                          x, direct_lstsq(a, b, cmplx=1),
-                                          rtol=25 * _eps_cast(a1.dtype),
-                                          atol=25 * _eps_cast(a1.dtype),
-                                          err_msg="driver: %s" % lapack_driver)
+                for lapack_driver in TestLstsq.lapack_drivers:
+                    for overwrite in (True, False):
+                        a = np.asarray(random([n, m]) + 1j*random([n, m]),
+                                       dtype=dtype)
+                        for i in range(m):
+                            a[i, i] = 20 * (0.1 + a[i, i])
+                        for i in range(2):
+                            b = np.asarray(random([n, 3]), dtype=dtype)
+                            # Store values in case they are overwritten
+                            # later
+                            a1 = a.copy()
+                            b1 = b.copy()
+                            out = lstsq(a1, b1,
+                                        lapack_driver=lapack_driver,
+                                        overwrite_a=overwrite,
+                                        overwrite_b=overwrite)
+                            x = out[0]
+                            r = out[2]
+                            assert_(r == m, 'expected efficient rank %s, '
+                                    'got %s' % (m, r))
+                            assert_allclose(
+                                      x, direct_lstsq(a, b, cmplx=1),
+                                      rtol=25 * _eps_cast(a1.dtype),
+                                      atol=25 * _eps_cast(a1.dtype),
+                                      err_msg="driver: %s" % lapack_driver)
 
     def test_check_finite(self):
         with suppress_warnings() as sup:
@@ -1236,44 +1175,31 @@ class TestLstsq(object):
             sup.filter(RuntimeWarning,
                        "internal gelsd driver lwork query error,.*"
                        "Falling back to 'gelss' driver.")
-            for dtype in REAL_DTYPES:
-                a = np.array(((1, 20), (-30, 4)), dtype=dtype)
-                for bt in (((1, 0), (0, 1)), (1, 0),
-                           ((2, 1), (-30, 4))):
-                    for lapack_driver in TestLstsq.lapack_drivers:
-                            for overwrite in (True, False):
-                                for check_finite in (True, False):
-                                    b = np.array(bt, dtype=dtype)
-                                    # Store values in case they are overwritten
-                                    # later
-                                    a1 = a.copy()
-                                    b1 = b.copy()
-                                    try:
-                                        out = lstsq(a1, b1,
-                                                    lapack_driver=lapack_driver,
-                                                    check_finite=check_finite,
-                                                    overwrite_a=overwrite,
-                                                    overwrite_b=overwrite)
-                                    except LstsqLapackError:
-                                        if lapack_driver is None:
-                                            mesg = (
-                                                'LstsqLapackError raised with '
-                                                'lapack_driver being None.')
-                                            raise AssertionError(mesg)
-                                        else:
-                                            # can't proceed,
-                                            # skip to the next iteration
-                                            continue
-                                    x = out[0]
-                                    r = out[2]
-                                    assert_(r == 2,
-                                            'expected efficient rank 2, '
-                                            'got %s' % r)
-                                    assert_allclose(
-                                          dot(a, x), b,
-                                          rtol=25 * _eps_cast(a.dtype),
-                                          atol=25 * _eps_cast(a.dtype),
-                                          err_msg="driver: %s" % lapack_driver)
+
+        at = np.array(((1, 20), (-30, 4)))
+        for dtype, bt, lapack_driver, overwrite, check_finite in \
+            itertools.product(REAL_DTYPES,
+                              (((1, 0), (0, 1)), (1, 0), ((2, 1), (-30, 4))),
+                              TestLstsq.lapack_drivers,
+                              (True, False),
+                              (True, False)):
+
+            a = at.astype(dtype)
+            b = np.array(bt, dtype=dtype)
+            # Store values in case they are overwritten
+            # later
+            a1 = a.copy()
+            b1 = b.copy()
+            out = lstsq(a1, b1, lapack_driver=lapack_driver,
+                        check_finite=check_finite, overwrite_a=overwrite,
+                        overwrite_b=overwrite)
+            x = out[0]
+            r = out[2]
+            assert_(r == 2, 'expected efficient rank 2, got %s' % r)
+            assert_allclose(dot(a, x), b,
+                            rtol=25 * _eps_cast(a.dtype),
+                            atol=25 * _eps_cast(a.dtype),
+                            err_msg="driver: %s" % lapack_driver)
 
     def test_zero_size(self):
         for a_shape, b_shape in (((0, 2), (0,)),
@@ -1373,6 +1299,28 @@ class TestPinvSymmetric(object):
         assert_array_almost_equal(np.dot(a, a_pinv), np.eye(3))
 
 
+def test_pinv_pinv2_comparison():  # As reported in gh-8861
+    I_6 = np.eye(6)
+    Ts = np.diag([-1] * 4 + [-2], k=-1) + np.diag([-2] + [-1] * 4, k=1)
+    T = I_6 + Ts
+    A = 25 * (np.kron(I_6, T) + np.kron(Ts, I_6))
+
+    Ap, Ap2 = pinv(A), pinv2(A)
+
+    tol = 1e-11
+    assert_allclose(A @ Ap @ A - A, A @ Ap2 @ A - A, rtol=0., atol=tol)
+    assert_allclose(Ap @ A @ Ap - Ap, Ap2 @ A @ Ap2 - Ap2, rtol=0., atol=tol)
+
+
+@pytest.mark.parametrize('scale', (1e-20, 1., 1e20))
+@pytest.mark.parametrize('pinv_', (pinv, pinvh, pinv2))
+def test_auto_rcond(scale, pinv_):
+    x = np.array([[1, 0], [0, 1e-10]]) * scale
+    expected = np.diag(1. / np.diag(x))
+    x_inv = pinv_(x)
+    assert_allclose(x_inv, expected)
+
+
 class TestVectorNorms(object):
 
     def test_types(self):
@@ -1654,4 +1602,3 @@ class TestMatrix_Balance(object):
             ip[p] = np.arange(A.shape[0])
             assert_allclose(y, np.diag(s)[ip, :])
             assert_allclose(solve(y, A).dot(y), x)
-
