@@ -3,12 +3,14 @@
 from __future__ import division, print_function, absolute_import
 
 from math import ceil, log
+import operator
 import warnings
 
 import numpy as np
 from numpy.fft import irfft, fft, ifft
 from scipy.special import sinc
-from scipy.linalg import toeplitz, hankel, pinv
+from scipy.linalg import (toeplitz, hankel, solve, LinAlgError, LinAlgWarning,
+                          lstsq)
 from scipy._lib.six import string_types
 
 from . import sigtools
@@ -17,11 +19,24 @@ __all__ = ['kaiser_beta', 'kaiser_atten', 'kaiserord',
            'firwin', 'firwin2', 'remez', 'firls', 'minimum_phase']
 
 
+def _get_fs(fs, nyq):
+    """
+    Utility for replacing the argument 'nyq' (with default 1) with 'fs'.
+    """
+    if nyq is None and fs is None:
+        fs = 2
+    elif nyq is not None:
+        if fs is not None:
+            raise ValueError("Values cannot be given for both 'nyq' and 'fs'.")
+        fs = 2*nyq
+    return fs
+
+
 # Some notes on function parameters:
 #
-# `cutoff` and `width` are given as a numbers between 0 and 1.  These
-# are relative frequencies, expressed as a fraction of the Nyquist rate.
-# For example, if the Nyquist rate is 2KHz, then width=0.15 is a width
+# `cutoff` and `width` are given as numbers between 0 and 1.  These are
+# relative frequencies, expressed as a fraction of the Nyquist frequency.
+# For example, if the Nyquist frequency is 2 KHz, then width=0.15 is a width
 # of 300 Hz.
 #
 # The `order` of a FIR filter is one less than the number of taps.
@@ -49,6 +64,17 @@ def kaiser_beta(a):
     References
     ----------
     Oppenheim, Schafer, "Discrete-Time Signal Processing", p.475-476.
+
+    Examples
+    --------
+    Suppose we want to design a lowpass filter, with 65 dB attenuation
+    in the stop band.  The Kaiser window parameter to be used in the
+    window method is computed by `kaiser_beta(65)`:
+
+    >>> from scipy.signal import kaiser_beta
+    >>> kaiser_beta(65)
+    6.20426
+
     """
     if a > 50:
         beta = 0.1102 * (a - 8.7)
@@ -73,7 +99,8 @@ def kaiser_atten(numtaps, width):
         The number of taps in the FIR filter.
     width : float
         The desired width of the transition region between passband and
-        stopband (or, in general, at any discontinuity) for the filter.
+        stopband (or, in general, at any discontinuity) for the filter,
+        expressed as a fraction of the Nyquist frequency.
 
     Returns
     -------
@@ -83,6 +110,19 @@ def kaiser_atten(numtaps, width):
     See Also
     --------
     kaiserord, kaiser_beta
+
+    Examples
+    --------
+    Suppose we want to design a FIR filter using the Kaiser window method
+    that will have 211 taps and a transition width of 9 Hz for a signal that
+    is sampled at 480 Hz.  Expressed as a fraction of the Nyquist frequency,
+    the width is 9/(0.5*480) = 0.0375.  The approximate attenuation (in dB)
+    is computed as follows:
+
+    >>> from scipy.signal import kaiser_atten
+    >>> kaiser_atten(211, 0.0375)
+    64.48099630593983
+
     """
     a = 2.285 * (numtaps - 1) * np.pi * width + 7.95
     return a
@@ -90,23 +130,36 @@ def kaiser_atten(numtaps, width):
 
 def kaiserord(ripple, width):
     """
-    Design a Kaiser window to limit ripple and width of transition region.
+    Determine the filter window parameters for the Kaiser window method.
+
+    The parameters returned by this function are generally used to create
+    a finite impulse response filter using the window method, with either
+    `firwin` or `firwin2`.
 
     Parameters
     ----------
     ripple : float
-        Positive number specifying maximum ripple in passband (dB) and minimum
-        ripple in stopband.
+        Upper bound for the deviation (in dB) of the magnitude of the
+        filter's frequency response from that of the desired filter (not
+        including frequencies in any transition intervals).  That is, if w
+        is the frequency expressed as a fraction of the Nyquist frequency,
+        A(w) is the actual frequency response of the filter and D(w) is the
+        desired frequency response, the design requirement is that::
+
+            abs(A(w) - D(w))) < 10**(-ripple/20)
+
+        for 0 <= w <= 1 and w not in a transition interval.
     width : float
-        Width of transition region (normalized so that 1 corresponds to pi
-        radians / sample).
+        Width of transition region, normalized so that 1 corresponds to pi
+        radians / sample.  That is, the frequency is expressed as a fraction
+        of the Nyquist frequency.
 
     Returns
     -------
     numtaps : int
-        The length of the kaiser window.
+        The length of the Kaiser window.
     beta : float
-        The beta parameter for the kaiser window.
+        The beta parameter for the Kaiser window.
 
     See Also
     --------
@@ -126,6 +179,73 @@ def kaiserord(ripple, width):
     ----------
     Oppenheim, Schafer, "Discrete-Time Signal Processing", p.475-476.
 
+    Examples
+    --------
+    We will use the Kaiser window method to design a lowpass FIR filter
+    for a signal that is sampled at 1000 Hz.
+
+    We want at least 65 dB rejection in the stop band, and in the pass
+    band the gain should vary no more than 0.5%.
+
+    We want a cutoff frequency of 175 Hz, with a transition between the
+    pass band and the stop band of 24 Hz.  That is, in the band [0, 163],
+    the gain varies no more than 0.5%, and in the band [187, 500], the
+    signal is attenuated by at least 65 dB.
+
+    >>> from scipy.signal import kaiserord, firwin, freqz
+    >>> import matplotlib.pyplot as plt
+    >>> fs = 1000.0
+    >>> cutoff = 175
+    >>> width = 24
+
+    The Kaiser method accepts just a single parameter to control the pass
+    band ripple and the stop band rejection, so we use the more restrictive
+    of the two.  In this case, the pass band ripple is 0.005, or 46.02 dB,
+    so we will use 65 dB as the design parameter.
+
+    Use `kaiserord` to determine the length of the filter and the
+    parameter for the Kaiser window.
+
+    >>> numtaps, beta = kaiserord(65, width/(0.5*fs))
+    >>> numtaps
+    167
+    >>> beta
+    6.20426
+
+    Use `firwin` to create the FIR filter.
+
+    >>> taps = firwin(numtaps, cutoff, window=('kaiser', beta),
+    ...               scale=False, nyq=0.5*fs)
+
+    Compute the frequency response of the filter.  ``w`` is the array of
+    frequencies, and ``h`` is the corresponding complex array of frequency
+    responses.
+
+    >>> w, h = freqz(taps, worN=8000)
+    >>> w *= 0.5*fs/np.pi  # Convert w to Hz.
+
+    Compute the deviation of the magnitude of the filter's response from
+    that of the ideal lowpass filter.  Values in the transition region are
+    set to ``nan``, so they won't appear in the plot.
+
+    >>> ideal = w < cutoff  # The "ideal" frequency response.
+    >>> deviation = np.abs(np.abs(h) - ideal)
+    >>> deviation[(w > cutoff - 0.5*width) & (w < cutoff + 0.5*width)] = np.nan
+
+    Plot the deviation.  A close look at the left end of the stop band shows
+    that the requirement for 65 dB attenuation is violated in the first lobe
+    by about 0.125 dB.  This is not unusual for the Kaiser window method.
+
+    >>> plt.plot(w, 20*np.log10(np.abs(deviation)))
+    >>> plt.xlim(0, 0.5*fs)
+    >>> plt.ylim(-90, -60)
+    >>> plt.grid(alpha=0.25)
+    >>> plt.axhline(-65, color='r', ls='--', alpha=0.3)
+    >>> plt.xlabel('Frequency (Hz)')
+    >>> plt.ylabel('Deviation from ideal (dB)')
+    >>> plt.title('Lowpass Filter Frequency Response')
+    >>> plt.show()
+
     """
     A = abs(ripple)  # in case somebody is confused as to what's meant
     if A < 8:
@@ -142,7 +262,7 @@ def kaiserord(ripple, width):
 
 
 def firwin(numtaps, cutoff, width=None, window='hamming', pass_zero=True,
-           scale=True, nyq=1.0):
+           scale=True, nyq=None, fs=None):
     """
     FIR filter design using the window method.
 
@@ -150,33 +270,37 @@ def firwin(numtaps, cutoff, width=None, window='hamming', pass_zero=True,
     filter.  The filter will have linear phase; it will be Type I if
     `numtaps` is odd and Type II if `numtaps` is even.
 
-    Type II filters always have zero response at the Nyquist rate, so a
+    Type II filters always have zero response at the Nyquist frequency, so a
     ValueError exception is raised if firwin is called with `numtaps` even and
-    having a passband whose right end is at the Nyquist rate.
+    having a passband whose right end is at the Nyquist frequency.
 
     Parameters
     ----------
     numtaps : int
         Length of the filter (number of coefficients, i.e. the filter
-        order + 1).  `numtaps` must be even if a passband includes the
+        order + 1).  `numtaps` must be odd if a passband includes the
         Nyquist frequency.
     cutoff : float or 1D array_like
-        Cutoff frequency of filter (expressed in the same units as `nyq`)
+        Cutoff frequency of filter (expressed in the same units as `fs`)
         OR an array of cutoff frequencies (that is, band edges). In the
         latter case, the frequencies in `cutoff` should be positive and
-        monotonically increasing between 0 and `nyq`.  The values 0 and
-        `nyq` must not be included in `cutoff`.
+        monotonically increasing between 0 and `fs/2`.  The values 0 and
+        `fs/2` must not be included in `cutoff`.
     width : float or None, optional
         If `width` is not None, then assume it is the approximate width
-        of the transition region (expressed in the same units as `nyq`)
+        of the transition region (expressed in the same units as `fs`)
         for use in Kaiser FIR filter design.  In this case, the `window`
         argument is ignored.
     window : string or tuple of string and parameter values, optional
         Desired window to use. See `scipy.signal.get_window` for a list
         of windows and required parameters.
-    pass_zero : bool, optional
+    pass_zero : {True, False, 'bandpass', 'lowpass', 'highpass', 'bandstop'}, optional
         If True, the gain at the frequency 0 (i.e. the "DC gain") is 1.
-        Otherwise the DC gain is 0.
+        If False, the DC gain is 0. Can also be a string argument for the
+        desired filter type (equivalent to ``btype`` in IIR design functions).
+
+        .. versionadded:: 1.3.0
+           Support for string arguments.
     scale : bool, optional
         Set to True to scale the coefficients so that the frequency
         response is exactly unity at a certain frequency.
@@ -184,13 +308,17 @@ def firwin(numtaps, cutoff, width=None, window='hamming', pass_zero=True,
 
         - 0 (DC) if the first passband starts at 0 (i.e. pass_zero
           is True)
-        - `nyq` (the Nyquist rate) if the first passband ends at
-          `nyq` (i.e the filter is a single band highpass filter);
+        - `fs/2` (the Nyquist frequency) if the first passband ends at
+          `fs/2` (i.e the filter is a single band highpass filter);
           center of first passband otherwise
 
     nyq : float, optional
-        Nyquist frequency.  Each frequency in `cutoff` must be between 0
-        and `nyq`.
+        *Deprecated.  Use `fs` instead.*  This is the Nyquist frequency.
+        Each frequency in `cutoff` must be between 0 and `nyq`. Default
+        is 1.
+    fs : float, optional
+        The sampling frequency of the signal.  Each frequency in `cutoff`
+        must be between 0 and ``fs/2``.  Default is 2.
 
     Returns
     -------
@@ -201,11 +329,11 @@ def firwin(numtaps, cutoff, width=None, window='hamming', pass_zero=True,
     ------
     ValueError
         If any value in `cutoff` is less than or equal to 0 or greater
-        than or equal to `nyq`, if the values in `cutoff` are not strictly
+        than or equal to ``fs/2``, if the values in `cutoff` are not strictly
         monotonically increasing, or if `numtaps` is even but a passband
         includes the Nyquist frequency.
 
-    See also
+    See Also
     --------
     firwin2
     firls
@@ -254,11 +382,12 @@ def firwin(numtaps, cutoff, width=None, window='hamming', pass_zero=True,
     >>> signal.firwin(numtaps, [f1, f2, f3, f4], pass_zero=False)
     array([ 0.04890915,  0.91284326,  0.04890915])
 
-    """
-
+    """  # noqa: E501
     # The major enhancements to this function added in November 2010 were
     # developed by Tom Krauss (see ticket #902).
 
+    nyq = 0.5 * _get_fs(fs, nyq)
+
     cutoff = np.atleast_1d(cutoff) / float(nyq)
 
     # Check for invalid input.
@@ -269,7 +398,7 @@ def firwin(numtaps, cutoff, width=None, window='hamming', pass_zero=True,
         raise ValueError("At least one cutoff frequency must be given.")
     if cutoff.min() <= 0 or cutoff.max() >= 1:
         raise ValueError("Invalid cutoff frequency: frequencies must be "
-                         "greater than 0 and less than nyq.")
+                         "greater than 0 and less than fs/2.")
     if np.any(np.diff(cutoff) <= 0):
         raise ValueError("Invalid cutoff frequencies: the frequencies "
                          "must be strictly increasing.")
@@ -281,10 +410,39 @@ def firwin(numtaps, cutoff, width=None, window='hamming', pass_zero=True,
         beta = kaiser_beta(atten)
         window = ('kaiser', beta)
 
+    if isinstance(pass_zero, str):
+        if pass_zero in ('bandstop', 'lowpass'):
+            if pass_zero == 'lowpass':
+                if cutoff.size != 1:
+                    raise ValueError('cutoff must have one element if '
+                                     'pass_zero=="lowpass", got %s'
+                                     % (cutoff.shape,))
+            elif cutoff.size <= 1:
+                raise ValueError('cutoff must have at least two elements if '
+                                 'pass_zero=="bandstop", got %s'
+                                 % (cutoff.shape,))
+            pass_zero = True
+        elif pass_zero in ('bandpass', 'highpass'):
+            if pass_zero == 'highpass':
+                if cutoff.size != 1:
+                    raise ValueError('cutoff must have one element if '
+                                     'pass_zero=="highpass", got %s'
+                                     % (cutoff.shape,))
+            elif cutoff.size <= 1:
+                raise ValueError('cutoff must have at least two elements if '
+                                 'pass_zero=="bandpass", got %s'
+                                 % (cutoff.shape,))
+            pass_zero = False
+        else:
+            raise ValueError('pass_zero must be True, False, "bandpass", '
+                             '"lowpass", "highpass", or "bandstop", got '
+                             '%s' % (pass_zero,))
+    pass_zero = bool(operator.index(pass_zero))  # ensure bool-like
+
     pass_nyquist = bool(cutoff.size & 1) ^ pass_zero
     if pass_nyquist and numtaps % 2 == 0:
         raise ValueError("A filter with an even number of coefficients must "
-                         "have zero response at the Nyquist rate.")
+                         "have zero response at the Nyquist frequency.")
 
     # Insert 0 and/or 1 at the ends of cutoff so that the length of cutoff
     # is even, and each pair in cutoff corresponds to passband.
@@ -328,8 +486,8 @@ def firwin(numtaps, cutoff, width=None, window='hamming', pass_zero=True,
 #
 # Rewritten by Warren Weckesser, 2010.
 
-def firwin2(numtaps, freq, gain, nfreqs=None, window='hamming', nyq=1.0,
-            antisymmetric=False):
+def firwin2(numtaps, freq, gain, nfreqs=None, window='hamming', nyq=None,
+            antisymmetric=False, fs=None):
     """
     FIR filter design using the window method.
 
@@ -344,11 +502,11 @@ def firwin2(numtaps, freq, gain, nfreqs=None, window='hamming', nyq=1.0,
         `nfreqs`.
     freq : array_like, 1D
         The frequency sampling points. Typically 0.0 to 1.0 with 1.0 being
-        Nyquist.  The Nyquist frequency can be redefined with the argument
-        `nyq`.
+        Nyquist.  The Nyquist frequency is half `fs`.
         The values in `freq` must be nondecreasing.  A value can be repeated
         once to implement a discontinuity.  The first value in `freq` must
-        be 0, and the last value must be `nyq`.
+        be 0, and the last value must be ``fs/2``. Values 0 and ``fs/2`` must
+        not be repeated.
     gain : array_like
         The filter gains at the frequency sampling points. Certain
         constraints to gain values, depending on the filter type, are applied,
@@ -364,11 +522,14 @@ def firwin2(numtaps, freq, gain, nfreqs=None, window='hamming', nyq=1.0,
         `scipy.signal.get_window` for the complete list of possible values.
         If None, no window function is applied.
     nyq : float, optional
-        Nyquist frequency.  Each frequency in `freq` must be between 0 and
-        `nyq` (inclusive).
+        *Deprecated.  Use `fs` instead.*  This is the Nyquist frequency.
+        Each frequency in `freq` must be between 0 and `nyq`.  Default is 1.
     antisymmetric : bool, optional
         Whether resulting impulse response is symmetric/antisymmetric.
         See Notes for more details.
+    fs : float, optional
+        The sampling frequency of the signal.  Each frequency in `cutoff`
+        must be between 0 and ``fs/2``.  Default is 2.
 
     Returns
     -------
@@ -429,6 +590,7 @@ def firwin2(numtaps, freq, gain, nfreqs=None, window='hamming', nyq=1.0,
     [-0.02286961 -0.06362756  0.57310236  0.57310236 -0.06362756 -0.02286961]
 
     """
+    nyq = 0.5 * _get_fs(fs, nyq)
 
     if len(freq) != len(gain):
         raise ValueError('freq and gain must be of same length.')
@@ -439,13 +601,17 @@ def firwin2(numtaps, freq, gain, nfreqs=None, window='hamming', nyq=1.0,
                          (numtaps, nfreqs))
 
     if freq[0] != 0 or freq[-1] != nyq:
-        raise ValueError('freq must start with 0 and end with `nyq`.')
+        raise ValueError('freq must start with 0 and end with fs/2.')
     d = np.diff(freq)
     if (d < 0).any():
         raise ValueError('The values in freq must be nondecreasing.')
     d2 = d[:-1] + d[1:]
     if (d2 == 0).any():
         raise ValueError('A value in freq must not occur more than twice.')
+    if freq[1] == 0:
+        raise ValueError('Value 0 must not be repeated in freq')
+    if freq[-2] == nyq:
+        raise ValueError('Value fs/2 must not be repeated in freq')
 
     if antisymmetric:
         if numtaps % 2 == 0:
@@ -460,22 +626,31 @@ def firwin2(numtaps, freq, gain, nfreqs=None, window='hamming', nyq=1.0,
 
     if ftype == 2 and gain[-1] != 0.0:
         raise ValueError("A Type II filter must have zero gain at the "
-                         "Nyquist rate.")
+                         "Nyquist frequency.")
     elif ftype == 3 and (gain[0] != 0.0 or gain[-1] != 0.0):
         raise ValueError("A Type III filter must have zero gain at zero "
-                         "and Nyquist rates.")
+                         "and Nyquist frequencies.")
     elif ftype == 4 and gain[0] != 0.0:
-        raise ValueError("A Type IV filter must have zero gain at zero rate.")
+        raise ValueError("A Type IV filter must have zero gain at zero "
+                         "frequency.")
 
     if nfreqs is None:
         nfreqs = 1 + 2 ** int(ceil(log(numtaps, 2)))
 
-    # Tweak any repeated values in freq so that interp works.
-    eps = np.finfo(float).eps
-    for k in range(len(freq)):
-        if k < len(freq) - 1 and freq[k] == freq[k + 1]:
-            freq[k] = freq[k] - eps
-            freq[k + 1] = freq[k + 1] + eps
+    if (d == 0).any():
+        # Tweak any repeated values in freq so that interp works.
+        freq = np.array(freq, copy=True)
+        eps = np.finfo(float).eps * nyq
+        for k in range(len(freq) - 1):
+            if freq[k] == freq[k + 1]:
+                freq[k] = freq[k] - eps
+                freq[k + 1] = freq[k + 1] + eps
+        # Check if freq is strictly increasing after tweak
+        d = np.diff(freq)
+        if (d <= 0).any():
+            raise ValueError("freq cannot contain numbers that are too close "
+                             "(within eps * (fs/2): "
+                             "{}) to a repeated value".format(eps))
 
     # Linearly interpolate the desired response on a uniform mesh `x`.
     x = np.linspace(0.0, nyq, nfreqs)
@@ -509,8 +684,8 @@ def firwin2(numtaps, freq, gain, nfreqs=None, window='hamming', nyq=1.0,
     return out
 
 
-def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
-          maxiter=25, grid_density=16):
+def remez(numtaps, bands, desired, weight=None, Hz=None, type='bandpass',
+          maxiter=25, grid_density=16, fs=None):
     """
     Calculate the minimax optimal filter using the Remez exchange algorithm.
 
@@ -525,9 +700,9 @@ def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
         The desired number of taps in the filter. The number of taps is
         the number of terms in the filter, or the filter order plus one.
     bands : array_like
-        A monotonic sequence containing the band edges in Hz.
+        A monotonic sequence containing the band edges.
         All elements must be non-negative and less than half the sampling
-        frequency as given by `Hz`.
+        frequency as given by `fs`.
     desired : array_like
         A sequence half the size of bands containing the desired gain
         in each of the specified bands.
@@ -535,6 +710,7 @@ def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
         A relative weighting to give to each band region. The length of
         `weight` has to be half the length of `bands`.
     Hz : scalar, optional
+        *Deprecated.  Use `fs` instead.*
         The sampling frequency in Hz. Default is 1.
     type : {'bandpass', 'differentiator', 'hilbert'}, optional
         The type of filter:
@@ -552,6 +728,8 @@ def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
     grid_density : int, optional
         Grid density. The dense grid used in `remez` is of size
         ``(numtaps + 1) * grid_density``. Default is 16.
+    fs : float, optional
+        The sampling frequency of the signal.  Default is 1.
 
     Returns
     -------
@@ -578,23 +756,90 @@ def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
 
     Examples
     --------
-    We want to construct a filter with a passband at 0.2-0.4 Hz, and
-    stop bands at 0-0.1 Hz and 0.45-0.5 Hz. Note that this means that the
-    behavior in the frequency ranges between those bands is unspecified and
-    may overshoot.
+    In these examples `remez` gets used creating a bandpass, bandstop, lowpass
+    and highpass filter.  The used parameters are the filter order, an array
+    with according frequency boundaries, the desired attenuation values and the
+    sampling frequency.  Using `freqz` the corresponding frequency response
+    gets calculated and plotted.
 
     >>> from scipy import signal
-    >>> bpass = signal.remez(72, [0, 0.1, 0.2, 0.4, 0.45, 0.5], [0, 1, 0])
-    >>> freq, response = signal.freqz(bpass)
-    >>> ampl = np.abs(response)
-
     >>> import matplotlib.pyplot as plt
-    >>> fig = plt.figure()
-    >>> ax1 = fig.add_subplot(111)
-    >>> ax1.semilogy(freq/(2*np.pi), ampl, 'b-')  # freq in Hz
+
+    >>> def plot_response(fs, w, h, title):
+    ...     "Utility function to plot response functions"
+    ...     fig = plt.figure()
+    ...     ax = fig.add_subplot(111)
+    ...     ax.plot(0.5*fs*w/np.pi, 20*np.log10(np.abs(h)))
+    ...     ax.set_ylim(-40, 5)
+    ...     ax.set_xlim(0, 0.5*fs)
+    ...     ax.grid(True)
+    ...     ax.set_xlabel('Frequency (Hz)')
+    ...     ax.set_ylabel('Gain (dB)')
+    ...     ax.set_title(title)
+
+    This example shows a steep low pass transition according to the small
+    transition width and high filter order:
+
+    >>> fs = 22050.0       # Sample rate, Hz
+    >>> cutoff = 8000.0    # Desired cutoff frequency, Hz
+    >>> trans_width = 100  # Width of transition from pass band to stop band, Hz
+    >>> numtaps = 400      # Size of the FIR filter.
+    >>> taps = signal.remez(numtaps, [0, cutoff, cutoff + trans_width, 0.5*fs], [1, 0], Hz=fs)
+    >>> w, h = signal.freqz(taps, [1], worN=2000)
+    >>> plot_response(fs, w, h, "Low-pass Filter")
+
+    This example shows a high pass filter:
+
+    >>> fs = 22050.0       # Sample rate, Hz
+    >>> cutoff = 2000.0    # Desired cutoff frequency, Hz
+    >>> trans_width = 250  # Width of transition from pass band to stop band, Hz
+    >>> numtaps = 125      # Size of the FIR filter.
+    >>> taps = signal.remez(numtaps, [0, cutoff - trans_width, cutoff, 0.5*fs],
+    ...                     [0, 1], Hz=fs)
+    >>> w, h = signal.freqz(taps, [1], worN=2000)
+    >>> plot_response(fs, w, h, "High-pass Filter")
+
+    For a signal sampled with 22 kHz a bandpass filter with a pass band of 2-5
+    kHz gets calculated using the Remez algorithm.  The transition width is 260
+    Hz and the filter order 10:
+
+    >>> fs = 22000.0         # Sample rate, Hz
+    >>> band = [2000, 5000]  # Desired pass band, Hz
+    >>> trans_width = 260    # Width of transition from pass band to stop band, Hz
+    >>> numtaps = 10        # Size of the FIR filter.
+    >>> edges = [0, band[0] - trans_width, band[0], band[1],
+    ...          band[1] + trans_width, 0.5*fs]
+    >>> taps = signal.remez(numtaps, edges, [0, 1, 0], Hz=fs)
+    >>> w, h = signal.freqz(taps, [1], worN=2000)
+    >>> plot_response(fs, w, h, "Band-pass Filter")
+
+    It can be seen that for this bandpass filter, the low order leads to higher
+    ripple and less steep transitions.  There is very low attenuation in the
+    stop band and little overshoot in the pass band.  Of course the desired
+    gain can be better approximated with a higher filter order.
+
+    The next example shows a bandstop filter. Because of the high filter order
+    the transition is quite steep:
+
+    >>> fs = 20000.0         # Sample rate, Hz
+    >>> band = [6000, 8000]  # Desired stop band, Hz
+    >>> trans_width = 200    # Width of transition from pass band to stop band, Hz
+    >>> numtaps = 175        # Size of the FIR filter.
+    >>> edges = [0, band[0] - trans_width, band[0], band[1], band[1] + trans_width, 0.5*fs]
+    >>> taps = signal.remez(numtaps, edges, [1, 0, 1], Hz=fs)
+    >>> w, h = signal.freqz(taps, [1], worN=2000)
+    >>> plot_response(fs, w, h, "Band-stop Filter")
+
     >>> plt.show()
 
     """
+    if Hz is None and fs is None:
+        fs = 1.0
+    elif Hz is not None:
+        if fs is not None:
+            raise ValueError("Values cannot be given for both 'Hz' and 'fs'.")
+        fs = Hz
+
     # Convert type
     try:
         tnum = {'bandpass': 1, 'differentiator': 2, 'hilbert': 3}[type]
@@ -607,11 +852,11 @@ def remez(numtaps, bands, desired, weight=None, Hz=1, type='bandpass',
         weight = [1] * len(desired)
 
     bands = np.asarray(bands).copy()
-    return sigtools._remez(numtaps, bands, desired, weight, tnum, Hz,
+    return sigtools._remez(numtaps, bands, desired, weight, tnum, fs,
                            maxiter, grid_density)
 
 
-def firls(numtaps, bands, desired, weight=None, nyq=1.):
+def firls(numtaps, bands, desired, weight=None, nyq=None, fs=None):
     """
     FIR filter design using least-squares error minimization.
 
@@ -638,8 +883,12 @@ def firls(numtaps, bands, desired, weight=None, nyq=1.):
         the least squares problem. `weight` has to be half the size of
         `bands`.
     nyq : float, optional
+        *Deprecated.  Use `fs` instead.*
         Nyquist frequency. Each frequency in `bands` must be between 0
-        and `nyq` (inclusive).
+        and `nyq` (inclusive).  Default is 1.
+    fs : float, optional
+        The sampling frequency of the signal. Each frequency in `bands`
+        must be between 0 and ``fs/2`` (inclusive).  Default is 2.
 
     Returns
     -------
@@ -690,21 +939,23 @@ def firls(numtaps, bands, desired, weight=None, nyq=1.):
     >>> from scipy import signal
     >>> import matplotlib.pyplot as plt
     >>> fig, axs = plt.subplots(2)
-    >>> nyq = 5.  # Hz
+    >>> fs = 10.0  # Hz
     >>> desired = (0, 0, 1, 1, 0, 0)
     >>> for bi, bands in enumerate(((0, 1, 2, 3, 4, 5), (0, 1, 2, 4, 4.5, 5))):
-    ...     fir_firls = signal.firls(73, bands, desired, nyq=nyq)
-    ...     fir_remez = signal.remez(73, bands, desired[::2], Hz=2 * nyq)
-    ...     fir_firwin2 = signal.firwin2(73, bands, desired, nyq=nyq)
+    ...     fir_firls = signal.firls(73, bands, desired, fs=fs)
+    ...     fir_remez = signal.remez(73, bands, desired[::2], fs=fs)
+    ...     fir_firwin2 = signal.firwin2(73, bands, desired, fs=fs)
     ...     hs = list()
     ...     ax = axs[bi]
     ...     for fir in (fir_firls, fir_remez, fir_firwin2):
     ...         freq, response = signal.freqz(fir)
-    ...         hs.append(ax.semilogy(nyq*freq/(np.pi), np.abs(response))[0])
-    ...     for band, gains in zip(zip(bands[::2], bands[1::2]), zip(desired[::2], desired[1::2])):
+    ...         hs.append(ax.semilogy(0.5*fs*freq/np.pi, np.abs(response))[0])
+    ...     for band, gains in zip(zip(bands[::2], bands[1::2]),
+    ...                            zip(desired[::2], desired[1::2])):
     ...         ax.semilogy(band, np.maximum(gains, 1e-7), 'k--', linewidth=2)
     ...     if bi == 0:
-    ...         ax.legend(hs, ('firls', 'remez', 'firwin2'), loc='lower center', frameon=False)
+    ...         ax.legend(hs, ('firls', 'remez', 'firwin2'),
+    ...                   loc='lower center', frameon=False)
     ...     else:
     ...         ax.set_xlabel('Frequency (Hz)')
     ...     ax.grid(True)
@@ -714,6 +965,8 @@ def firls(numtaps, bands, desired, weight=None, nyq=1.):
     >>> plt.show()
 
     """  # noqa
+    nyq = 0.5 * _get_fs(fs, nyq)
+
     numtaps = int(numtaps)
     if numtaps % 2 == 0 or numtaps < 1:
         raise ValueError("numtaps must be odd and >= 1")
@@ -726,6 +979,8 @@ def firls(numtaps, bands, desired, weight=None, nyq=1.):
     bands = np.asarray(bands).flatten() / nyq
     if len(bands) % 2 != 0:
         raise ValueError("bands must contain frequency pairs.")
+    if (bands < 0).any() or (bands > 1).any():
+        raise ValueError("bands must be between 0 and 1 relative to Nyquist")
     bands.shape = (-1, 2)
 
     # check remaining params
@@ -778,7 +1033,7 @@ def firls(numtaps, bands, desired, weight=None, nyq=1.):
 
     # Now for b(n) we have that:
     #     b(n) = 1/π ∫ W(ω)D(ω)cos(nω)dω (over 0->π)
-    # Using our nomalization ω=πf and with a constant weight W over each
+    # Using our normalization ω=πf and with a constant weight W over each
     # interval and a linear term for D(ω) we get (over each f1->f2 interval):
     #     b(n) = W ∫ (mf+c)cos(πnf)df
     #          = f(mf+c)sin(πnf)/πnf + mf**2 cos(nπf)/(πnf)**2
@@ -793,8 +1048,20 @@ def firls(numtaps, bands, desired, weight=None, nyq=1.):
     b[1:] += m * np.cos(n[1:] * np.pi * bands) / (np.pi * n[1:]) ** 2
     b = np.dot(np.diff(b, axis=2)[:, :, 0], weight)
 
-    # Now we can solve the equation (use pinv because Q can be rank deficient)
-    a = np.dot(pinv(Q), b)
+    # Now we can solve the equation
+    try:  # try the fast way
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            a = solve(Q, b, sym_pos=True, check_finite=False)
+        for ww in w:
+            if (ww.category == LinAlgWarning and
+                    str(ww.message).startswith('Ill-conditioned matrix')):
+                raise LinAlgError(str(ww.message))
+    except LinAlgError:  # in case Q is rank deficient
+        # This is faster than pinvh, even though we don't explicitly use
+        # the symmetry here. gelsy was faster than gelsd and gelss in
+        # some non-exhaustive tests.
+        a = lstsq(Q, b, lapack_driver='gelsy')[0]
 
     # make coefficients symmetric (linear phase)
     coeffs = np.hstack((a[:0:-1], 2 * a[0], a[1:]))
@@ -935,7 +1202,7 @@ def minimum_phase(h, method='homomorphic', n_fft=None):
            doi: 10.1109/ICASSP.1999.756179
     .. [2] X. Chen and T. W. Parks, "Design of optimal minimum phase FIR
            filters by direct factorization," Signal Processing,
-           vol. 10, no. 4, pp. 369–383, Jun. 1986.
+           vol. 10, no. 4, pp. 369-383, Jun. 1986.
     .. [3] T. Saramaki, "Finite Impulse Response Filter Design," in
            Handbook for Digital Signal Processing, chapter 4,
            New York: Wiley-Interscience, 1993.
